﻿// ***********************************************************************
// Assembly         : DynamicClash.MHRK
// Author           : mariuroe
// Created          : 04-08-2014
//
// Last Modified By : mariuroe
// Last Modified On : 04-16-2014
// ***********************************************************************
// <copyright file="ObservableProgress.cs" >
//     Copyright (c) . All rights reserved.
// </copyright>
// <summary></summary>
// ***********************************************************************

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Threading;
using Autodesk.Navisworks.Api;

/// <summary>
/// The Models namespace includes all the models and interfaces in the MVVM arcitechture. 
/// </summary>
namespace DynamicRelaxationClashVerification.Models
{
    /// <summary>
    /// The abstract ObservableProgress class contains methods and fields to give a class
    /// oppertunity to observe a progress in the class. It is part of the observable 
    /// observer technique.
    /// </summary>
    public abstract class ObservableProgress
    {
        /// <summary>
        /// The list of all the listeners
        /// </summary>
        private List<ProgressListener> _listeners = new List<ProgressListener>();
        /// <summary>
        /// Gets or sets the progress minimum.
        /// </summary>
        /// <value>The progress minimum.</value>
        public double ProgressMin { get; set; }
        /// <summary>
        /// Gets or sets the progress maximum.
        /// </summary>
        /// <value>The progress maximum.</value>
        public double ProgressMax { get; set; }
        /// <summary>
        /// Adds the progress listener to this observable prosess.
        /// </summary>
        /// <param name="listener">The listener.</param>
        public void AddProgress(ProgressListener listener)
        {
            if(!_listeners.Contains(listener))
                _listeners.Add(listener);
        }
        /// <summary>
        /// Removes the progress listener from the observable prosess
        /// </summary>
        /// <param name="listener">The listener.</param>
        public void RemoveProgress(ProgressListener listener)
        {
            if (_listeners.Contains(listener))
                _listeners.Remove(listener);
        }
        /// <summary>
        /// Fires the progress changed: Tells all the listeners about
        /// the change in the observable progress.
        /// </summary>
        /// <param name="progressValue">The progress value.</param>
        public void FireProgressChanged(double progressValue)
        {
            DoEvents();
            double value = progressValue / (ProgressMax - ProgressMin) * 100;

            foreach (ProgressListener listener in _listeners)
            {
                listener.ProgressChanged(this, ProgressMin, ProgressMax, value);
            }
        }
        /// <summary>
        /// Thread handling for updating UI. Using of dispathcer. See 
        /// Microsoft documentation for more information.
        /// </summary>
        public static void DoEvents()
        {
            System.Windows.Application.Current.Dispatcher.Invoke(
            DispatcherPriority.Background, new Action(delegate { }));
        }
        /// <summary>
        /// The _result listeners
        /// </summary>
        private List<ResultListener> _resultListeners = new List<ResultListener>();
        /// <summary>
        /// Adds the result listener.
        /// </summary>
        /// <param name="listener">The listener.</param>
        public void AddResultListener(ResultListener listener)
        {
            if (!_resultListeners.Contains(listener))
                _resultListeners.Add(listener);
        }
        /// <summary>
        /// Removes the result listener.
        /// </summary>
        /// <param name="listener">The listener.</param>
        public void RemoveResultListener(ResultListener listener)
        {
            if (_resultListeners.Contains(listener))
                _resultListeners.Remove(listener);
        }
        /// <summary>
        /// Fires the result changed.
        /// </summary>
        /// <param name="velocityPoint">The velocity point.</param>
        /// <param name="accelerationPoint">The acceleration point.</param>
        /// <param name="forcePoint">The force point.</param>
        public void FireResultChanged(Point2D velocityPoint, Point2D accelerationPoint, Point2D forcePoint)
        {
            DoEvents();
            foreach (ResultListener listener in _resultListeners)
            {
                listener.ResultChanged(this, velocityPoint, accelerationPoint, forcePoint);
            }
        }
    }
}
