﻿// ***********************************************************************
// Assembly         : DynamicClash.MHRK
// Author           : mariuroe
// Created          : 03-07-2014
//
// Last Modified By : mariuroe
// Last Modified On : 04-16-2014
// ***********************************************************************
// <copyright file="ControlViewModel.cs" >
//     Copyright (c) . All rights reserved.
// </copyright>
// <summary></summary>
// ***********************************************************************

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using Autodesk.Navisworks.Api;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using System.Collections.ObjectModel;

using ComBridge = Autodesk.Navisworks.Api.ComApi.ComApiBridge;
using COMApi = Autodesk.Navisworks.Api.Interop.ComApi;
using Autodesk.Navisworks.Api.Interop.ComApi;
using System.Windows.Media;
using System.Windows;
using System.Windows.Forms;
using System.Windows.Threading;

using DynamicRelaxationClashVerification.Models;
using DynamicRelaxationClashVerification.Models.DataModels;
using System.ComponentModel;
using Autodesk.Navisworks.Api.Clash;
using System.Threading;


/// <summary>
/// The ViewModels namespace.
/// </summary>
namespace DynamicRelaxationClashVerification.ViewModels
{
    /// <summary>
    /// Class ControlViewModel.
    /// </summary>
    public class ControlViewModel : ViewModelBase, ProgressListener
    {
        /// <summary>
        /// Gets the update selection sets command.
        /// </summary>
        /// <value>The update selection sets command.</value>
        public ICommand UpdateSelectionSetsCmd { get; private set; }
        /// <summary>
        /// Gets the add view point command.
        /// </summary>
        /// <value>The add view point command.</value>
        public ICommand AddViewPointCmd { get; private set; }
        /// <summary>
        /// Gets the create animation command.
        /// </summary>
        /// <value>The create animation command.</value>
        public ICommand CreateAnimationCmd { get; private set; }
        /// <summary>
        /// Gets the run phase one command.
        /// </summary>
        /// <value>The run phase one command.</value>
        public ICommand RunPhaseOneCmd { get; private set; }
        /// <summary>
        /// Gets the run phase two command.
        /// </summary>
        /// <value>The run phase two command.</value>
        public ICommand RunPhaseTwoCmd { get; private set; }
        /// <summary>
        /// Gets the run path factory command.
        /// </summary>
        /// <value>The run path factory command.</value>
        public ICommand RunPathFactoryCmd { get; private set; }
        /// <summary>
        /// Gets the show popup canvas command.
        /// </summary>
        /// <value>The show popup canvas command.</value>
        public ICommand ShowPopupCanvasCmd { get; private set; }
        /// <summary>
        /// Gets the revert start position command.
        /// </summary>
        /// <value>The revert start position command.</value>
        public ICommand RevertStartPositionCmd { get; private set; }
        /// <summary>
        /// Gets the run cancel command.
        /// </summary>
        /// <value>The run cancel command.</value>
        public ICommand RunCancelCmd { get; private set; }

        public ICommand RunCancelOpimizationCmd { get; private set; }

        public ICommand RunDynamicMovementCmd { get; private set; }

        //The current opened Navisworks document/ file.
        /// <summary>
        /// The document
        /// </summary>
        protected Document doc = Autodesk.Navisworks.Api.Application.MainDocument;

        /// <summary>
        /// The dynamic clash verification
        /// </summary>
        private DynamicClashVerification DynamicClashVerification;

        /// <summary>
        /// The _progress value
        /// </summary>
        private double _progressValue = 0;
        /// <summary>
        /// Gets or sets the progress value.
        /// </summary>
        /// <value>The progress value.</value>
        public double ProgressValue
        {
            get { return _progressValue; }
            set
            {
                _progressValue = value;
                RaisePropertyChanged(() => ProgressValue);
            }
        }

        /// <summary>
        /// The _progress minimum
        /// </summary>
        private double _progressMin = 0;
        /// <summary>
        /// Gets or sets the progress minimum.
        /// </summary>
        /// <value>The progress minimum.</value>
        public double ProgressMin
        {
            get { return _progressMin; }
            set
            {
                _progressMin = value;
                RaisePropertyChanged(() => ProgressMin);
            }
        }

        /// <summary>
        /// The _progress maximum
        /// </summary>
        private double _progressMax = 0;
        /// <summary>
        /// Gets or sets the progress maximum.
        /// </summary>
        /// <value>The progress maximum.</value>
        public double ProgressMax
        {
            get { return _progressMax; }
            set
            {
                _progressMax = value;
                RaisePropertyChanged(() => ProgressMax);
            }
        }

        /// <summary>
        /// Progresses the changed.
        /// </summary>
        /// <param name="obs">The obs.</param>
        /// <param name="min">The minimum.</param>
        /// <param name="max">The maximum.</param>
        /// <param name="value">The value.</param>
        public void ProgressChanged(ObservableProgress obs, double min, double max, double value)
        {
            _observableProgress = obs;
            ProgressMin = min;
            ProgressMax = max;
            ProgressValue = value;
        }

        /// <summary>
        /// The _observable progress
        /// </summary>
        private ObservableProgress _observableProgress;

        /// <summary>
        /// Initializes a new instance of the <see cref="ControlViewModel"/> class.
        /// </summary>
        public ControlViewModel()
        {

            //RelayCommands for the buttons in the ControlView.

            UpdateSelectionSetsCmd = new RelayCommand(UpdateSelectionSets, () => true);
            AddViewPointCmd = new RelayCommand(AddViewPointToAnimation, () => true);
            CreateAnimationCmd = new RelayCommand(RunCreateAnimation, () => true);
            RunPhaseOneCmd = new RelayCommand(RunPhaseOne, () => true);
            RunPhaseTwoCmd = new RelayCommand(RunPhaseTwo, () => true);
            RunPathFactoryCmd = new RelayCommand(RunPathFactory, () => true);
            ShowPopupCanvasCmd = new RelayCommand(SetShowPopupCanvas, () => true);
            RevertStartPositionCmd = new RelayCommand(RevertStartPosition, () => true);
            RunCancelCmd = new RelayCommand(CancelVerification, () => true);
            RunDynamicMovementCmd = new RelayCommand(RunDynamicMovement, () => true);
            RunCancelOpimizationCmd = new RelayCommand(CancelOptimization, ()=>true);

            //Message passing with the DynamicClashVerificationViewModel
            MessengerInstance.Register<DynamicClashVerification>(this, "SelectedVerificationChanged", dCv => ChangeDynamicClashVerification(dCv));

            ClashTypeListString.Add("Hard Conservative");
            ClashTypeListString.Add("Hard");
            ClashTypeListString.Add("Clearance");
            ClashTypeListString.Add("Duplicate");
        }

        /// <summary>
        /// Runs the dynamic movement.
        /// </summary>
        private void RunDynamicMovement()
        {
            DynamicClashVerification.RunDynamicMovement();
        }

        private Visibility _verifyResultButtonVisibility  = Visibility.Collapsed;
        public Visibility VerifyResultButtonVisibility
        {
            get { return _verifyResultButtonVisibility; }
            set
            {
                _verifyResultButtonVisibility = value;
                RaisePropertyChanged(() => VerifyResultButtonVisibility);
            }
        }

        /// <summary>
        /// Reverts the start position.
        /// </summary>
        private void RevertStartPosition()
        {
            DynamicClashVerification.RevertStartPosition();
            FillPathCollectionWithPoints(DynamicClashVerification.Path.CurrentPath);
        }

        /// <summary>
        /// Changes the dynamic clash verification.
        /// </summary>
        /// <param name="dCv">The d cv.</param>
        private void ChangeDynamicClashVerification(DynamicClashVerification dCv)
        {
            DynamicClashVerification = dCv;

            if (dCv != null)
            {
                CurrentValidationObject = DynamicClashVerification.ValidationObjectItem;
                CurrentBoundaryObject = DynamicClashVerification.BoundaryGeometryItem;
                CurrentPath = DynamicClashVerification.PathItem;
                PhaseOneResultCollection = DynamicClashVerification.PhaseOneResultCollection;
                MovementPrecision = DynamicClashVerification.MovementPrecision;

                if (DynamicClashVerification.ValidationObjectItem != null)
                {
                    AxlePosition = DynamicClashVerification.ValidationObject.AxlePosition;
                    AxleCenterLength = DynamicClashVerification.ValidationObject.AxleCenterDistance;
                    IsAxleInFront = (DynamicClashVerification.ValidationObject.AxlePosition == AxlePosition.Front ? true : false);
                    IsAxleInRear = (DynamicClashVerification.ValidationObject.AxlePosition == AxlePosition.Rear ? true : false);
                }
                else
                {
                    AxlePosition = AxlePosition.Center;
                    AxleCenterLength = 0;
                    IsAxleInFront = false;
                    IsAxleInRear = false;
                }
            }

            if (dCv != null)
            {
                IsControlPanelEnable = true;
            }
            else
            {
                IsControlPanelEnable = false;
            }
        }

        /// <summary>
        /// Clears the dynamic clash verification.
        /// </summary>
        private void ClearDynamicClashVerification()
        {
            ProgressMin = 0;
            ProgressMax = 100;
            ProgressValue = 0;

            AxlePosition = AxlePosition.Center;
            AxleCenterLength = 0;
            IsAxleInFront = false;
            IsAxleInRear = false;

            MovementPrecision = 200;
        }

        /// <summary>
        /// The _clash type
        /// </summary>
        private ClashTestType _clashType = ClashTestType.HardConservative;
        /// <summary>
        /// Gets or sets the type of the clash.
        /// </summary>
        /// <value>The type of the clash.</value>
        public ClashTestType ClashType
        {
            get { return _clashType; }
            set
            {
                _clashType = value;
                RaisePropertyChanged(() => ClashType);
            }
        }

        /// <summary>
        /// The _clash tolerance
        /// </summary>
        private double _clashTolerance = 0.0001;
        /// <summary>
        /// Gets or sets the clash tolerance.
        /// </summary>
        /// <value>The clash tolerance.</value>
        public double ClashTolerance
        {
            get { return _clashTolerance; }
            set
            {
                _clashTolerance = value;
                RaisePropertyChanged(() => ClashTolerance);
            }

        }

        /// <summary>
        /// The _axle position
        /// </summary>
        private AxlePosition _axlePosition = AxlePosition.Center;
        /// <summary>
        /// Gets or sets the axle position.
        /// </summary>
        /// <value>The axle position.</value>
        public AxlePosition AxlePosition { get; set; }

        /// <summary>
        /// The _axle center length
        /// </summary>
        private double _axleCenterLength = 0;
        /// <summary>
        /// Gets or sets the length of the axle center.
        /// </summary>
        /// <value>The length of the axle center.</value>
        public double AxleCenterLength
        {
            get { return _axleCenterLength; }
            set
            {
                _axleCenterLength = value;
                RaisePropertyChanged(() => AxleCenterLength);
            }
        }

        /// <summary>
        /// The _is axle in front
        /// </summary>
        private bool _isAxleInFront = false;
        /// <summary>
        /// Gets or sets a value indicating whether this instance is axle in front.
        /// </summary>
        /// <value><c>true</c> if this instance is axle in front; otherwise, <c>false</c>.</value>
        public bool IsAxleInFront
        {
            get { return _isAxleInFront; }
            set
            {
                _isAxleInFront = value;
                _axlePosition = AxlePosition.Front;
                RaisePropertyChanged(() => IsAxleInFront);
            }
        }

        /// <summary>
        /// The _is axle in rear
        /// </summary>
        private bool _isAxleInRear = false;
        /// <summary>
        /// Gets or sets a value indicating whether this instance is axle in rear.
        /// </summary>
        /// <value><c>true</c> if this instance is axle in rear; otherwise, <c>false</c>.</value>
        public bool IsAxleInRear
        {
            get { return _isAxleInRear; }
            set
            {
                _isAxleInRear = value;
                _axlePosition = AxlePosition.Rear;
                RaisePropertyChanged(() => IsAxleInRear);
            }
        }

        /// <summary>
        /// The _clash type list string
        /// </summary>
        private List<string> _clashTypeListString = new List<string>();
        /// <summary>
        /// Gets or sets the clash type list string.
        /// </summary>
        /// <value>The clash type list string.</value>
        public List<string> ClashTypeListString
        {
            get { return _clashTypeListString; }
            set
            {
                _clashTypeListString = value;
                RaisePropertyChanged(() => ClashTypeListString);
            }
        }

        /// <summary>
        /// The _selected clash type
        /// </summary>
        private string _selectedClashType = "Hard Conservative";
        /// <summary>
        /// Gets or sets the type of the selected clash.
        /// </summary>
        /// <value>The type of the selected clash.</value>
        public string SelectedClashType
        {
            get { return _selectedClashType; }
            set
            {
                _selectedClashType = value;
                SetClashType(_selectedClashType);
                RaisePropertyChanged(() => SelectedClashType);
            }
        }

        /// <summary>
        /// Sets the type of the clash.
        /// </summary>
        /// <param name="type">The type.</param>
        private void SetClashType(string type)
        {
            if(type.Equals("Clearance"))
                ClashType = ClashTestType.Clearance;
            else if (type.Equals("Duplicate"))
                ClashType = ClashTestType.Duplicate;
            else if (type.Equals("Hard"))
                ClashType = ClashTestType.Hard;
            else if (type.Equals("Hard Conservative"))
                ClashType = ClashTestType.HardConservative;
        }

        /// <summary>
        /// The _movement precision
        /// </summary>
        private int _movementPrecision = 200;
        /// <summary>
        /// Gets or sets the movement precision.
        /// </summary>
        /// <value>The movement precision.</value>
        public int MovementPrecision
        {
            get { return _movementPrecision; }
            set
            {
                _movementPrecision = value;
                DynamicClashVerification.MovementPrecision = _movementPrecision;
                RaisePropertyChanged(() => MovementPrecision);
            }
        }

        /// <summary>
        /// The _is control panel enable
        /// </summary>
        private bool _isControlPanelEnable = false;
        /// <summary>
        /// Gets or sets a value indicating whether this instance is control panel enable.
        /// </summary>
        /// <value><c>true</c> if this instance is control panel enable; otherwise, <c>false</c>.</value>
        public bool IsControlPanelEnable
        {
            get { return _isControlPanelEnable; }
            set
            {
                _isControlPanelEnable = value;
                RaisePropertyChanged(() => IsControlPanelEnable);
            }
        }

        /*
         * Properties and method for setting the current
         * validation and boundary object, based on the selected
         * section sets.
         */
        /// <summary>
        /// The _current validation object string
        /// </summary>
        private string _currentValidationObjectString;
        /// <summary>
        /// Gets or sets the current validation object string.
        /// </summary>
        /// <value>The current validation object string.</value>
        public string CurrentValidationObjectString
        {
            get { return _currentValidationObjectString; }
            set
            {
                _currentValidationObjectString = value;
                SetCurrentItem(value, false);
                RaisePropertyChanged(() => CurrentValidationObjectString);
            }
        }
        /// <summary>
        /// The _current boundary object string
        /// </summary>
        private string _currentBoundaryObjectString;
        /// <summary>
        /// Gets or sets the current boundary object string.
        /// </summary>
        /// <value>The current boundary object string.</value>
        public string CurrentBoundaryObjectString
        {
            get { return _currentBoundaryObjectString; }
            set
            {
                _currentBoundaryObjectString = value;
                SetCurrentItem(value, true);
                RaisePropertyChanged(() => CurrentBoundaryObjectString);
            }
        }

        /// <summary>
        /// Sets the current item.
        /// </summary>
        /// <param name="displayName">The display name.</param>
        /// <param name="isBoundary">if set to <c>true</c> [is boundary].</param>
        private void SetCurrentItem(string displayName, bool isBoundary)
        {
            foreach (SavedItem item in _selectionSets)
            {
                if (item.DisplayName.Equals(displayName))
                {
                    if (isBoundary)
                    {
                        CurrentBoundaryObject = (item as SelectionSet).ExplicitModelItems;
                    }
                    else
                    {
                        CurrentValidationObject = (item as SelectionSet).ExplicitModelItems;
                    }
                }
            }
        }

        /// <summary>
        /// The _current validation object
        /// </summary>
        private ModelItemCollection _currentValidationObject;
        /// <summary>
        /// Gets or sets the current validation object.
        /// </summary>
        /// <value>The current validation object.</value>
        public ModelItemCollection CurrentValidationObject
        {
            get { return _currentValidationObject; }
            set
            {
                _currentValidationObject = value;
                RaisePropertyChanged(() => CurrentValidationObject);
            }
        }
        /// <summary>
        /// The _current boundary object
        /// </summary>
        private ModelItemCollection _currentBoundaryObject;
        /// <summary>
        /// Gets or sets the current boundary object.
        /// </summary>
        /// <value>The current boundary object.</value>
        public ModelItemCollection CurrentBoundaryObject
        {
            get { return _currentBoundaryObject; }
            set
            {
                _currentBoundaryObject = value;
                RaisePropertyChanged(() => CurrentBoundaryObject);
            }
        }

        /*
         * Properties and methods for setting current
         * ideal path for DR.
         */
        /// <summary>
        /// The _current path string
        /// </summary>
        private string _currentPathString;
        /// <summary>
        /// Gets or sets the current path string.
        /// </summary>
        /// <value>The current path string.</value>
        public string CurrentPathString
        {
            get { return _currentPathString; }
            set
            {
                _currentPathString = value;
                SetCurrentPath(value);
                RaisePropertyChanged(() => CurrentPathString);
            }
        }
        /// <summary>
        /// The _current path
        /// </summary>
        private ModelItemCollection _currentPath;
        /// <summary>
        /// Gets or sets the current path.
        /// </summary>
        /// <value>The current path.</value>
        public ModelItemCollection CurrentPath
        {
            get { return _currentPath; }
            set
            {
                _currentPath = value;
                RaisePropertyChanged(() => CurrentPath);
            }
        }

        /// <summary>
        /// Sets the current path.
        /// </summary>
        /// <param name="pathName">Name of the path.</param>
        private void SetCurrentPath(string pathName)
        {
            foreach (SavedItem item in _selectionSets)
            {
                if (item.DisplayName.Equals(pathName))
                {
                    CurrentPath = (item as SelectionSet).ExplicitModelItems;
                }
            }
        }

        /*
         * Method and properties for getting and updating the
         * Selection Sets from Navisworks through 
         * the .NET API.
         */
        /// <summary>
        /// The _selection sets
        /// </summary>
        private List<SavedItem> _selectionSets = new List<SavedItem>();
        /// <summary>
        /// The _selection sets in string
        /// </summary>
        private ObservableCollection<string> _selectionSetsInString = new ObservableCollection<string>();
        /// <summary>
        /// Gets or sets the selection sets in string.
        /// </summary>
        /// <value>The selection sets in string.</value>
        public ObservableCollection<string> SelectionSetsInString
        {
            get 
            {
                return _selectionSetsInString; 
            }
            set
            {
                _selectionSetsInString = value;
                RaisePropertyChanged(() => SelectionSetsInString);
            }
        }
        /// <summary>
        /// Gets the selection set list.
        /// </summary>
        public void GetSelectionSetList()
        {
            try
            {
                _selectionSetsInString.Clear();
                _selectionSets.Clear();
                List<SavedItem> tempList = doc.SelectionSets.ToSavedItemCollection().ToList();
                foreach (SavedItem item in tempList)
                {
                    _selectionSetsInString.Add(item.DisplayName);
                    _selectionSets.Add(item);
                }
                SelectionSetsInString = _selectionSetsInString;
            }
            catch (Exception)
            {
                // Dd nothing
            }
        }
        /// <summary>
        /// Updates the selection sets.
        /// </summary>
        public void UpdateSelectionSets()
        {
            GetSelectionSetList();
        }

        //Method and property for animation using Viewpoints
        /// <summary>
        /// The _animation
        /// </summary>
        private Animation _animation = new Animation();
        /// <summary>
        /// Adds the view point to animation.
        /// </summary>
        public void AddViewPointToAnimation()
        {
            _animation.AddViewPointToAnimation();
        }
        /// <summary>
        /// Runs the create animation.
        /// </summary>
        public void RunCreateAnimation()
        {
            _animation.CreateAnimation();
        }

        //Method for handling UI and background threads
        /// <summary>
        /// Does the events.
        /// </summary>
        public static void DoEvents()
        {
            System.Windows.Application.Current.Dispatcher.Invoke(
            DispatcherPriority.Background, new Action(delegate { }));
        }

        public void ChangeStatus(Status status)
        {
            DynamicClashVerification.Status = status;
            MessengerInstance.Send(status, "StatusChanged");
        }

        //Method for running Phase one of verification. Collection to store results.
        public void CancelOptimization()
        {
            DynamicClashVerification.CancelOptimization();
        }

        /// <summary>
        /// The _phase one result collection
        /// </summary>
        private ObservableCollection<Clash3D> _phaseOneResultCollection = new ObservableCollection<Clash3D>();
        /// <summary>
        /// Gets or sets the phase one result collection.
        /// </summary>
        /// <value>The phase one result collection.</value>
        public ObservableCollection<Clash3D> PhaseOneResultCollection
        {
            get
            {
                return _phaseOneResultCollection;
            }
            set
            {
                _phaseOneResultCollection = value;
                RaisePropertyChanged(() => PhaseOneResultCollection);
            }
        }
        /// <summary>
        /// Runs the phase one.
        /// </summary>
        public void RunPhaseOne()
        {
            ChangeStatus(Status.Working);
            ProgressBarColor = System.Windows.Media.Color.FromRgb(18, 186, 0);

            DoEvents();
            DynamicClashVerification.SetBoundaryGeometry(CurrentBoundaryObject);
            DynamicClashVerification.SetValidationObject(CurrentValidationObject, AxleCenterLength, _axlePosition);
            DynamicClashVerification.ProgressListener = this;

            bool res = DynamicClashVerification.RunPhaseOneOfVerification();

            if (res == false && DynamicClashVerification.Status == Status.Cancelled)
            {
                ChangeStatus(Status.Cancelled);
                System.Windows.Forms.MessageBox.Show("The optimizing was cancelled!");
                ProgressBarColor = System.Windows.Media.Color.FromRgb(186, 0, 0);
            }
            else
            {
                PhaseOneResultCollection = DynamicClashVerification.PhaseOneResultCollection;
                FillOptimizedPathCollectionWithPoints(DynamicClashVerification.DynamicMovement.RealPath);
                ChangeStatus(Status.Optimized);
            }
        }

        //Method for running Phase two of verification. Collection to store results.
        /// <summary>
        /// The _phase two result collection
        /// </summary>
        private ObservableCollection<Point3D> _phaseTwoResultCollection = new ObservableCollection<Point3D>();
        /// <summary>
        /// Gets or sets the phase two result collection.
        /// </summary>
        /// <value>The phase two result collection.</value>
        public ObservableCollection<Point3D> PhaseTwoResultCollection
        {
            get
            {
                return _phaseTwoResultCollection;
            }
            set
            {
                _phaseTwoResultCollection = value;
                RaisePropertyChanged(() => PhaseTwoResultCollection);
            }
        }
        /// <summary>
        /// Runs the phase two.
        /// </summary>
        public void RunPhaseTwo()
        {
            ChangeStatus(Status.Working);

            DoEvents();

            ProgressValue = 0;
            ProgressMax = 0;
            ProgressMin = 0;
            ProgressBarColor = System.Windows.Media.Color.FromRgb(18, 186, 0);

            DynamicClashVerification.ProgressListener = this;

            Path newPath = new Path();

            bool firstTime = false;
            if (DynamicClashVerification.PhaseOneResultRealPointList.Count() > 0)
            {
                newPath.CurrentPath = DynamicClashVerification.PhaseOneResultRealPointList;
                firstTime = false;
            }
            else if (DynamicClashVerification.PhaseTwoResultCollection != null && DynamicClashVerification.PhaseTwoResultCollection.Count() > 0)
            {
                //If phase one has not been run
                newPath.CurrentPath = DynamicClashVerification.Path.CurrentPath;
                DynamicClashVerification.Path.CurrentPath = DynamicClashVerification.PhaseTwoResultCollection;
                firstTime = false;
            }
            else
            {
                //If phase one has not been run
                newPath.CurrentPath = DynamicClashVerification.Path.CurrentPath;
                DynamicClashVerification.SetBoundaryGeometry(CurrentBoundaryObject);
                DynamicClashVerification.SetValidationObject(CurrentValidationObject, AxleCenterLength, _axlePosition);
                firstTime = true;
            }
            MessengerInstance.Send(DynamicClashVerification, "SelectedVerificationChanged");

            bool result = DynamicClashVerification.RunPhaseTwoOfVerification(newPath, firstTime);

            if (result)
            {
                ChangeStatus(Status.Finished);
                ProgressBarColor = System.Windows.Media.Color.FromRgb(18, 186, 0);
                FillFinalPathCollectionWithPoints(DynamicClashVerification.PhaseTwoResultCollection);
                VerifyResultButtonVisibility = Visibility.Visible;
            }
            else
            {
                if (DynamicClashVerification.Status == Status.Cancelled)
                {
                    ChangeStatus(Status.Cancelled);
                    System.Windows.Forms.MessageBox.Show("The verification was cancelled!");
                }
                else
                {
                    ChangeStatus(Status.Failed);
                    System.Windows.Forms.MessageBox.Show("The verification failed!");
                }
                ProgressBarColor = System.Windows.Media.Color.FromRgb(186, 0, 0);
            }
        }

        /// <summary>
        /// Cancels the verification.
        /// </summary>
        private void CancelVerification()
        {
            DynamicClashVerification.CancelVerification();
        }

        /// <summary>
        /// The _progress bar color
        /// </summary>
        private System.Windows.Media.Color _progressBarColor = System.Windows.Media.Color.FromRgb(18,186,0);
        /// <summary>
        /// Gets or sets the color of the progress bar.
        /// </summary>
        /// <value>The color of the progress bar.</value>
        public System.Windows.Media.Color ProgressBarColor
        {
            get { return _progressBarColor; }
            set
            {
                _progressBarColor = value;
                RaisePropertyChanged(() => ProgressBarColor);
            }
        }

        //Path factory generation methods and properties
        /// <summary>
        /// Runs the path factory.
        /// </summary>
        public void RunPathFactory()
        {
            ProgressValue = 0;
            ProgressMax = 0;
            ProgressMin = 0;
            ProgressBarColor = System.Windows.Media.Color.FromRgb(18, 186, 0);

            DynamicClashVerification.ProgressListener = this;

            DynamicClashVerification.SetPath(CurrentPath);
            IsPathSet = false;
            FillPathCollectionWithPoints(DynamicClashVerification.Path.CurrentPath);
        }

        /// <summary>
        /// The _is path set
        /// </summary>
        private bool _isPathSet = true;
        /// <summary>
        /// Gets or sets a value indicating whether this instance is path set.
        /// </summary>
        /// <value><c>true</c> if this instance is path set; otherwise, <c>false</c>.</value>
        public bool IsPathSet
        {
            get { return _isPathSet; }
            set
            {
                _isPathSet = value;
                RaisePropertyChanged(() => IsPathSet);
            }

        }

        /*
         * Method and properties for Path popup overview.
         */

        /// <summary>
        /// The _path collection
        /// </summary>
        private PointCollection _pathCollection = new PointCollection();
        /// <summary>
        /// Gets or sets the path collection.
        /// </summary>
        /// <value>The path collection.</value>
        public PointCollection PathCollection
        {
            get { return _pathCollection; }
            set
            {
                _pathCollection = value;
                RaisePropertyChanged(() => PathCollection);
            }
        }

        /// <summary>
        /// The _optimized path collection
        /// </summary>
        private PointCollection _optimizedPathCollection = new PointCollection();
        /// <summary>
        /// Gets or sets the optimized path collection.
        /// </summary>
        /// <value>The optimized path collection.</value>
        public PointCollection OptimizedPathCollection
        {
            get { return _optimizedPathCollection; }
            set
            {
                _optimizedPathCollection = value;
                RaisePropertyChanged(() => OptimizedPathCollection);
            }
        }

        /// <summary>
        /// The _final path collection
        /// </summary>
        private PointCollection _finalPathCollection = new PointCollection();
        /// <summary>
        /// Gets or sets the final path collection.
        /// </summary>
        /// <value>The final path collection.</value>
        public PointCollection FinalPathCollection
        {
            get { return _finalPathCollection; }
            set
            {
                _finalPathCollection = value;
                RaisePropertyChanged(() => FinalPathCollection);
            }
        }

        /// <summary>
        /// The _scaling
        /// </summary>
        private double _scaling;
        /// <summary>
        /// The _lagerst x
        /// </summary>
        private double _lagerstX;
        /// <summary>
        /// The _lagerst y
        /// </summary>
        private double _lagerstY;

        /// <summary>
        /// The _smallest x
        /// </summary>
        private double _smallestX;
        /// <summary>
        /// The _smallest y
        /// </summary>
        private double _smallestY;

        /// <summary>
        /// Fills the path collection with points.
        /// </summary>
        /// <param name="newPath">The new path.</param>
        private void FillPathCollectionWithPoints(List<Point3D> newPath)
        {
            _lagerstX = FindLargestValueInPointList(newPath, true);
            _lagerstY = FindLargestValueInPointList(newPath, false);

            _smallestX = FindSmallestValueInPointList(newPath, true);
            _smallestY = FindSmallestValueInPointList(newPath, false);

            _scaling = CalculateCanvasScalingFactor(_lagerstX-_smallestX, _lagerstY-_smallestY);

            PathCollection.Clear();
            PointCollection pathCollection = new PointCollection();
            CanvasWidth = (int)((newPath.First().X-_smallestX) * _scaling);
            CanvasHeight = (int)((newPath.First().Y-_smallestY) * _scaling);
            foreach(Point3D point in newPath)
            {
                pathCollection.Add(new Point((point.X-_smallestX) * _scaling, (point.Y-_smallestY) * _scaling));

                if ((point.X - _smallestX) * _scaling > CanvasWidth)
                    CanvasWidth = (int)((point.X - _smallestX) * _scaling);
                if ((point.Y - _smallestY) * _scaling > CanvasHeight)
                    CanvasHeight = (int)((point.Y - _smallestY) * _scaling);
            }

            CanvasWidth += 10;
            CanvasHeight += 10;

            PathCollection = pathCollection;

            //Subtract 7.5 because of centering drawing point
            StartPositionX = PathCollection.First().X-7.5;
            StartPositionY = PathCollection.First().Y-7.5;

            ShowFactoryPath = true;
            ShowFactoryPoints = true;
        }

        /// <summary>
        /// Fills the optimized path collection with points.
        /// </summary>
        /// <param name="newOptimizedPath">The new optimized path.</param>
        private void FillOptimizedPathCollectionWithPoints(List<Point3D> newOptimizedPath)
        {
            OptimizedPathCollection.Clear();
            PointCollection pathCollection = new PointCollection();

            int canvasWidth = (int)((newOptimizedPath.First().X - _smallestX) * _scaling);
            int canvasHeight = (int)((newOptimizedPath.First().Y - _smallestY) * _scaling);

            foreach (Point3D point in newOptimizedPath)
            {
                pathCollection.Add(new Point((point.X - _smallestX) * _scaling, (point.Y - _smallestY) * _scaling));

                if ((point.X - _smallestX) * _scaling > canvasWidth)
                    canvasWidth = (int)((point.X - _smallestX) * _scaling);
                if ((point.Y - _smallestY) * _scaling > canvasHeight)
                    canvasHeight = (int)((point.Y - _smallestY) * _scaling);

            }

            if (canvasHeight > CanvasHeight)
                CanvasHeight = canvasHeight;

            if (canvasWidth > CanvasWidth)
                CanvasWidth = canvasWidth;


            OptimizedPathCollection = pathCollection;

            ShowOptimizedPath = true;
            ShowOptimizedPoints = true;
        }

        private void FillFinalPathCollectionWithPoints(List<Point3D> newFinalPath)
        {
            FinalPathCollection.Clear();
            PointCollection pathCollection = new PointCollection();

            int canvasWidth = (int)((newFinalPath.First().X - _smallestX) * _scaling);
            int canvasHeight = (int)((newFinalPath.First().Y - _smallestY) * _scaling);

            foreach (Point3D point in newFinalPath)
            {
                pathCollection.Add(new Point((point.X - _smallestX) * _scaling, (point.Y - _smallestY) * _scaling));

                if ((point.X - _smallestX) * _scaling > canvasWidth)
                    canvasWidth = (int)((point.X - _smallestX) * _scaling);
                if ((point.Y - _smallestY) * _scaling > canvasHeight)
                    canvasHeight = (int)((point.Y - _smallestY) * _scaling);

            }

            if (canvasHeight > CanvasHeight)
                CanvasHeight = canvasHeight;

            if (canvasWidth > CanvasWidth)
                CanvasWidth = canvasWidth;


            FinalPathCollection = pathCollection;

            ShowFinalPath = true;
            ShowFinalPoints = true;
        }

        /// <summary>
        /// Finds the largest value in point list.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="isX">if set to <c>true</c> [is x].</param>
        /// <returns>System.Double.</returns>
        private double FindLargestValueInPointList(List<Point3D> list, bool isX)
        {
            double largest = 0;
            foreach (Point3D point in list)
            {
                if (isX && Math.Abs(point.X) > largest)
                    largest = Math.Abs(point.X);
                else if (isX == false && Math.Abs(point.Y) > largest)
                    largest = Math.Abs(point.Y);
            }
            return largest;
        }

        /// <summary>
        /// Finds the smallest value in point list.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="isX">if set to <c>true</c> [is x].</param>
        /// <returns>System.Double.</returns>
        private double FindSmallestValueInPointList(List<Point3D> list, bool isX)
        {
            
            double smallest;
            if (isX)
                smallest = list.First().X;
            else
                smallest = list.First().Y;

            foreach (Point3D point in list)
            {
                if (isX && Math.Abs(point.X) < smallest)
                    smallest = Math.Abs(point.X);
                else if (isX == false && Math.Abs(point.Y) < smallest)
                    smallest = Math.Abs(point.Y);
            }
            return smallest;
        }

        /// <summary>
        /// Calculates the canvas scaling factor.
        /// </summary>
        /// <param name="largestX">The largest x.</param>
        /// <param name="largestY">The largest y.</param>
        /// <returns>System.Double.</returns>
        private double CalculateCanvasScalingFactor(double largestX, double largestY)
        {
            int max = 300;

            if (max / largestX > max / largestY)
                return max / largestX;
            else
                return max / largestY;
        }

        /// <summary>
        /// The _canvas width
        /// </summary>
        private int _canvasWidth = 0;
        /// <summary>
        /// Gets or sets the width of the canvas.
        /// </summary>
        /// <value>The width of the canvas.</value>
        public int CanvasWidth
        {
            get { return _canvasWidth; }
            set
            {
                _canvasWidth = value;
                RaisePropertyChanged(() => CanvasWidth);
            }
        }

        /// <summary>
        /// The _canvas height
        /// </summary>
        private int _canvasHeight = 0;
        /// <summary>
        /// Gets or sets the height of the canvas.
        /// </summary>
        /// <value>The height of the canvas.</value>
        public int CanvasHeight
        {
            get { return _canvasHeight; }
            set
            {
                _canvasHeight = value;
                RaisePropertyChanged(() => CanvasHeight);
            }
        }

        /// <summary>
        /// The _start position x
        /// </summary>
        private double _startPositionX;
        /// <summary>
        /// Gets or sets the start position x.
        /// </summary>
        /// <value>The start position x.</value>
        public double StartPositionX
        {
            get { return _startPositionX; }
            set
            {
                _startPositionX = value;
                RaisePropertyChanged(() => StartPositionX);
            }
        }

        /// <summary>
        /// The _start position y
        /// </summary>
        private double _startPositionY;
        /// <summary>
        /// Gets or sets the start position y.
        /// </summary>
        /// <value>The start position y.</value>
        public double StartPositionY
        {
            get { return _startPositionY; }
            set
            {
                _startPositionY = value;
                RaisePropertyChanged(() => StartPositionY);
            }
        }

        /// <summary>
        /// The _show popup canvas
        /// </summary>
        private bool _showPopupCanvas = false;
        /// <summary>
        /// Gets or sets a value indicating whether [show popup canvas].
        /// </summary>
        /// <value><c>true</c> if [show popup canvas]; otherwise, <c>false</c>.</value>
        public bool ShowPopupCanvas
        {
            get { return _showPopupCanvas; }
            set
            {
                _showPopupCanvas = value;
                RaisePropertyChanged(() => ShowPopupCanvas);
            }
        }

        /// <summary>
        /// Sets the show popup canvas.
        /// </summary>
        public void SetShowPopupCanvas()
        {
            if (ShowPopupCanvas == true)
                ShowPopupCanvas = false;
            else
                ShowPopupCanvas = true;
        }


//Visibility for paths
        /// <summary>
        /// The _factory path visibility
        /// </summary>
        private Visibility _factoryPathVisibility = Visibility.Hidden;
        /// <summary>
        /// Gets or sets the factory path visibility.
        /// </summary>
        /// <value>The factory path visibility.</value>
        public Visibility FactoryPathVisibility 
        {
            get { return _factoryPathVisibility; }
            set
            {
                _factoryPathVisibility = value;
                RaisePropertyChanged(() => FactoryPathVisibility);
            }
        }

        /// <summary>
        /// The _factory points visibility
        /// </summary>
        private Visibility _factoryPointsVisibility = Visibility.Hidden;
        /// <summary>
        /// Gets or sets the factory points visibility.
        /// </summary>
        /// <value>The factory points visibility.</value>
        public Visibility FactoryPointsVisibility
        {
            get { return _factoryPointsVisibility; }
            set
            {
                _factoryPointsVisibility = value;
                RaisePropertyChanged(() => FactoryPointsVisibility);
            }
        }

        /// <summary>
        /// The _optimized path visibility
        /// </summary>
        private Visibility _optimizedPathVisibility = Visibility.Hidden;
        /// <summary>
        /// Gets or sets the optimized path visibility.
        /// </summary>
        /// <value>The optimized path visibility.</value>
        public Visibility OptimizedPathVisibility
        {
            get { return _optimizedPathVisibility; }
            set
            {
                _optimizedPathVisibility = value;
                RaisePropertyChanged(() => OptimizedPathVisibility);
            }
        }

        /// <summary>
        /// The _optimized points visibility
        /// </summary>
        private Visibility _optimizedPointsVisibility = Visibility.Hidden;
        /// <summary>
        /// Gets or sets the optimized points visibility.
        /// </summary>
        /// <value>The optimized points visibility.</value>
        public Visibility OptimizedPointsVisibility
        {
            get { return _optimizedPointsVisibility; }
            set
            {
                _optimizedPointsVisibility = value;
                RaisePropertyChanged(() => OptimizedPointsVisibility);
            }
        }

        /// <summary>
        /// The _d r optimized path visibility
        /// </summary>
        private Visibility _dROptimizedPathVisibility = Visibility.Hidden;
        /// <summary>
        /// Gets or sets the dr optimized path visibility.
        /// </summary>
        /// <value>The dr optimized path visibility.</value>
        public Visibility DROptimizedPathVisibility
        {
            get { return _dROptimizedPathVisibility; }
            set
            {
                _dROptimizedPathVisibility = value;
                RaisePropertyChanged(() => DROptimizedPathVisibility);
            }
        }

        /// <summary>
        /// The _d r optimized points visibility
        /// </summary>
        private Visibility _dROptimizedPointsVisibility = Visibility.Hidden;
        /// <summary>
        /// Gets or sets the dr optimized points visibility.
        /// </summary>
        /// <value>The dr optimized points visibility.</value>
        public Visibility DROptimizedPointsVisibility
        {
            get { return _dROptimizedPointsVisibility; }
            set
            {
                _dROptimizedPointsVisibility = value;
                RaisePropertyChanged(() => DROptimizedPointsVisibility);
            }
        }

        /// <summary>
        /// The _final path visibility
        /// </summary>
        private Visibility _finalPathVisibility = Visibility.Hidden;
        /// <summary>
        /// Gets or sets the final path visibility.
        /// </summary>
        /// <value>The final path visibility.</value>
        public Visibility FinalPathVisibility
        {
            get { return _finalPathVisibility; }
            set
            {
                _finalPathVisibility = value;
                RaisePropertyChanged(() => FinalPathVisibility);
            }
        }

        /// <summary>
        /// The _final points visibility
        /// </summary>
        private Visibility _finalPointsVisibility = Visibility.Hidden;
        /// <summary>
        /// Gets or sets the final points visibility.
        /// </summary>
        /// <value>The final points visibility.</value>
        public Visibility FinalPointsVisibility
        {
            get { return _finalPointsVisibility; }
            set
            {
                _finalPointsVisibility = value;
                RaisePropertyChanged(() => FinalPointsVisibility);
            }
        }
//CheckBox buttons
        /// <summary>
        /// The _show factory path
        /// </summary>
        private bool _showFactoryPath = false;
        /// <summary>
        /// Gets or sets a value indicating whether [show factory path].
        /// </summary>
        /// <value><c>true</c> if [show factory path]; otherwise, <c>false</c>.</value>
        public bool ShowFactoryPath
        {
            get { return _showFactoryPath; }
            set
            {
                _showFactoryPath = value;

                if (value == true)
                    FactoryPathVisibility = Visibility.Visible;
                else
                    FactoryPathVisibility = Visibility.Hidden;

                RaisePropertyChanged(() => ShowFactoryPath);
            }
        }

        /// <summary>
        /// The _show factory points
        /// </summary>
        private bool _showFactoryPoints = false;
        /// <summary>
        /// Gets or sets a value indicating whether [show factory points].
        /// </summary>
        /// <value><c>true</c> if [show factory points]; otherwise, <c>false</c>.</value>
        public bool ShowFactoryPoints
        {
            get { return _showFactoryPoints; }
            set
            {
                _showFactoryPoints = value;

                if (value == true)
                    FactoryPointsVisibility = Visibility.Visible;
                else
                    FactoryPointsVisibility = Visibility.Hidden;

                RaisePropertyChanged(() => ShowFactoryPoints);
            }
        }

        /// <summary>
        /// The _show optimized path
        /// </summary>
        private bool _showOptimizedPath = false;
        /// <summary>
        /// Gets or sets a value indicating whether [show optimized path].
        /// </summary>
        /// <value><c>true</c> if [show optimized path]; otherwise, <c>false</c>.</value>
        public bool ShowOptimizedPath
        {
            get { return _showOptimizedPath; }
            set
            {
                _showOptimizedPath = value;

                if (value == true)
                    OptimizedPathVisibility = Visibility.Visible;
                else
                    OptimizedPathVisibility = Visibility.Hidden;

                RaisePropertyChanged(() => ShowOptimizedPath);
            }
        }

        /// <summary>
        /// The _show optimized points
        /// </summary>
        private bool _showOptimizedPoints = false;
        /// <summary>
        /// Gets or sets a value indicating whether [show optimized points].
        /// </summary>
        /// <value><c>true</c> if [show optimized points]; otherwise, <c>false</c>.</value>
        public bool ShowOptimizedPoints
        {
            get { return _showOptimizedPoints; }
            set
            {
                _showOptimizedPoints = value;

                if (value == true)
                    OptimizedPointsVisibility = Visibility.Visible;
                else
                    OptimizedPointsVisibility = Visibility.Hidden;

                RaisePropertyChanged(() => ShowOptimizedPoints);
            }
        }

        /// <summary>
        /// The _show final path
        /// </summary>
        private bool _showFinalPath = false;
        /// <summary>
        /// Gets or sets a value indicating whether [show final path].
        /// </summary>
        /// <value><c>true</c> if [show final path]; otherwise, <c>false</c>.</value>
        public bool ShowFinalPath
        {
            get { return _showFinalPath; }
            set
            {
                _showFinalPath = value;

                if (value == true)
                    FinalPathVisibility = Visibility.Visible;
                else
                    FinalPathVisibility = Visibility.Hidden;

                RaisePropertyChanged(() => ShowFinalPath);
            }
        }

        /// <summary>
        /// The _show final points
        /// </summary>
        private bool _showFinalPoints = false;
        /// <summary>
        /// Gets or sets a value indicating whether [show final points].
        /// </summary>
        /// <value><c>true</c> if [show final points]; otherwise, <c>false</c>.</value>
        public bool ShowFinalPoints
        {
            get { return _showFinalPoints; }
            set
            {
                _showFinalPoints = value;

                if (value == true)
                    FinalPointsVisibility = Visibility.Visible;
                else
                    FinalPointsVisibility = Visibility.Hidden;

                RaisePropertyChanged(() => ShowFinalPoints);
            }
        }

    }
}
