﻿// ***********************************************************************
// Assembly         : DynamicClash.MHRK
// Author           : mariuroe
// Created          : 03-19-2014
//
// Last Modified By : mariuroe
// Last Modified On : 04-09-2014
// ***********************************************************************
// <copyright file="DynamicClashVerificationViewModel.cs" company="">
//     Copyright (c) . All rights reserved.
// </copyright>
// <summary></summary>
// ***********************************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GalaSoft.MvvmLight;
using System.Collections.ObjectModel;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;
using System.Windows;
using DynamicRelaxationClashVerification.Models;
using DynamicRelaxationClashVerification.Models.DataModels;
using SH = System.Windows.Shapes;
using System.Windows.Media;
using System.ComponentModel;

/// <summary>
/// The ViewModels namespace.
/// </summary>
namespace DynamicRelaxationClashVerification.ViewModels
{
    /// <summary>
    /// Class DynamicClashVerificationViewModel.
    /// </summary>
    public class DynamicClashVerificationViewModel : ViewModelBase
    {
        //ICommands for the buttons
        /// <summary>
        /// Gets the add dynamic clash verification command.
        /// </summary>
        /// <value>The add dynamic clash verification command.</value>
        public ICommand AddDynamicClashVerificationCmd { get; private set; }
        /// <summary>
        /// Gets the delete dynamic clash verification command.
        /// </summary>
        /// <value>The delete dynamic clash verification command.</value>
        public ICommand DeleteDynamicClashVerificationCmd { get; private set; }
        /// <summary>
        /// Gets the delete all dynamic clash verification command.
        /// </summary>
        /// <value>The delete all dynamic clash verification command.</value>
        public ICommand DeleteAllDynamicClashVerificationCmd { get; private set; }

        /// <summary>
        /// The _dynamic clash verifications
        /// </summary>
        private ObservableCollection<DynamicClashVerification> _dynamicClashVerifications = new ObservableCollection<DynamicClashVerification>();
        /// <summary>
        /// Gets or sets the dynamic clash verifications.
        /// </summary>
        /// <value>The dynamic clash verifications.</value>
        public ObservableCollection<DynamicClashVerification> DynamicClashVerifications 
        {
            get { return _dynamicClashVerifications; }
            set
            {
                _dynamicClashVerifications = value;
                RaisePropertyChanged(() => DynamicClashVerifications);
            }
        }

        /// <summary>
        /// The _verification name
        /// </summary>
        private string _verificationName = "No verification selected";
        /// <summary>
        /// Gets or sets the name of the verification.
        /// </summary>
        /// <value>The name of the verification.</value>
        public string VerificationName 
        {
            get { return _verificationName; }
            set
            {
                _verificationName = value;
                RaisePropertyChanged(() => VerificationName);
            }
        }

        /// <summary>
        /// The _selected verification
        /// </summary>
        private DynamicClashVerification _selectedVerification;
        /// <summary>
        /// Gets or sets the selected verification.
        /// </summary>
        /// <value>The selected verification.</value>
        public DynamicClashVerification SelectedVerification
        {
            get { return _selectedVerification; }
            set
            {
                _selectedVerification = value;
                RaisePropertyChanged(() => SelectedVerification);

                if (SelectedVerification != null)
                    VerificationName = SelectedVerification.Name;
                else
                    VerificationName = "No verification selected";

                SelectedVerificationChanged();
            }
        }

        //Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="DynamicClashVerificationViewModel"/> class.
        /// </summary>
        public DynamicClashVerificationViewModel()
        {
            if (!IsInDesignMode)
            {
                AddDynamicClashVerificationCmd = new RelayCommand(AddDynamicClashVerification, () => true);
                DeleteDynamicClashVerificationCmd = new RelayCommand(DeleteDynamicClashVerification, () => true);
                DeleteAllDynamicClashVerificationCmd = new RelayCommand(DeleteAllDynamicClashVerification, () => true);

                _dynamicClashVerificationVisibility = Visibility.Collapsed;
            }
            MessengerInstance.Register<Status>(this, "StatusChanged", status => ChangeStatus(status));
        }

        /// <summary>
        /// Changes the status.
        /// </summary>
        /// <param name="status">The status.</param>
        public void ChangeStatus(Status status)
        {
            SelectedVerification.Status = status;
            ObservableCollection<DynamicClashVerification> temp = new ObservableCollection<DynamicClashVerification>();
            foreach (DynamicClashVerification dyn in DynamicClashVerifications)
            {
                temp.Add(dyn);
            }
            DynamicClashVerifications = temp;
        }

        //Methods for adding and deleting items
        /// <summary>
        /// Adds the dynamic clash verification.
        /// </summary>
        public void AddDynamicClashVerification()
        {
            DynamicClashVerifications.Add(new DynamicClashVerification("Default DR name", "Dynamic Clash Verification using DR"));
            DynamicClashVerificationsChanged();
        }
        /// <summary>
        /// Deletes the dynamic clash verification.
        /// </summary>
        public void DeleteDynamicClashVerification()
        {
            for (int i = 0; i < DynamicClashVerifications.Count(); i++) 
            {
                if (DynamicClashVerifications.ElementAt(i).Equals(SelectedVerification))
                    DynamicClashVerifications.RemoveAt(i);
            }
            DynamicClashVerificationsChanged();
        }
        /// <summary>
        /// Deletes all dynamic clash verification.
        /// </summary>
        public void DeleteAllDynamicClashVerification()
        {
            DynamicClashVerifications.Clear();
            DynamicClashVerificationsChanged();
        }

        /// <summary>
        /// Dynamics the clash verifications changed.
        /// </summary>
        private void DynamicClashVerificationsChanged()
        {
            if (DynamicClashVerifications.Count() == 0)
            {
                DynamicClashVerificationVisibility = Visibility.Collapsed;
            }
            else
            {
                DynamicClashVerificationVisibility = Visibility.Visible;
            }
        }
        /// <summary>
        /// The _dynamic clash verification visibility
        /// </summary>
        private Visibility _dynamicClashVerificationVisibility;
        /// <summary>
        /// Gets or sets the dynamic clash verification visibility.
        /// </summary>
        /// <value>The dynamic clash verification visibility.</value>
        public Visibility DynamicClashVerificationVisibility
        {
            get { return _dynamicClashVerificationVisibility; }
            set 
            {
                _dynamicClashVerificationVisibility = value;
                RaisePropertyChanged(() => DynamicClashVerificationVisibility);
            }
        }

        /// <summary>
        /// Selecteds the verification changed.
        /// </summary>
        private void SelectedVerificationChanged()
        {
            MessengerInstance.Send(SelectedVerification, "SelectedVerificationChanged");
        }

        }

}
