﻿// ***********************************************************************
// Assembly         : DynamicClash.MHRK
// Author           : mariuroe
// Created          : 04-29-2014
//
// Last Modified By : mariuroe
// Last Modified On : 04-30-2014
// ***********************************************************************
// <copyright file="ResultViewModel.cs" company="">
//     Copyright (c) . All rights reserved.
// </copyright>
// <summary></summary>
// ***********************************************************************
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Autodesk.Navisworks.Api;
using System.Collections.ObjectModel;
using GalaSoft.MvvmLight;
using DynamicRelaxationClashVerification.Models;

/// <summary>
/// The ViewModels namespace.
/// </summary>
namespace DynamicRelaxationClashVerification.ViewModels
{
    /// <summary>
    /// Class ResultViewModel.
    /// </summary>
    public class ResultViewModel : ViewModelBase, ResultListener
    {
        /// <summary>
        /// The result
        /// </summary>
        private ObservableProgress result;

        /// <summary>
        /// Initializes a new instance of the <see cref="ResultViewModel"/> class.
        /// </summary>
        public ResultViewModel()
        {
            MessengerInstance.Register<DynamicClashVerification>(this, "SelectedVerificationChanged", dCv => ChangeDynamicClashVerification(dCv));
        }

        /// <summary>
        /// Gets or sets the dynamic clash verification.
        /// </summary>
        /// <value>The dynamic clash verification.</value>
        public DynamicClashVerification DynamicClashVerification { get; set; }
        /// <summary>
        /// Changes the dynamic clash verification.
        /// </summary>
        /// <param name="dCv">The d cv.</param>
        private void ChangeDynamicClashVerification(DynamicClashVerification dCv)
        {
            DynamicClashVerification = dCv;
            if(dCv != null)
                DynamicClashVerification.ResultListener = this;
        }

        /// <summary>
        /// The _velocity collection
        /// </summary>
        private ObservableCollection<Point2D> _velocityCollection = new ObservableCollection<Point2D>();
        /// <summary>
        /// Gets or sets the velocity collection.
        /// </summary>
        /// <value>The velocity collection.</value>
        public ObservableCollection<Point2D> VelocityCollection
        {
            get { return _velocityCollection; }
            set
            {
                _velocityCollection = value;
                RaisePropertyChanged(() => VelocityCollection);
            }
        }

        /// <summary>
        /// The _acceleration collection
        /// </summary>
        private ObservableCollection<Point2D> _accelerationCollection = new ObservableCollection<Point2D>();
        /// <summary>
        /// Gets or sets the acceleration collection.
        /// </summary>
        /// <value>The acceleration collection.</value>
        public ObservableCollection<Point2D> AccelerationCollection
        {
            get { return _accelerationCollection; }
            set
            {
                _accelerationCollection = value;
                RaisePropertyChanged(() => AccelerationCollection);
            }
        }

        /// <summary>
        /// The _force collection
        /// </summary>
        private ObservableCollection<Point2D> _forceCollection = new ObservableCollection<Point2D>();
        /// <summary>
        /// Gets or sets the force collection.
        /// </summary>
        /// <value>The force collection.</value>
        public ObservableCollection<Point2D> ForceCollection
        {
            get { return _forceCollection; }
            set
            {
                _forceCollection = value;
                RaisePropertyChanged(() => ForceCollection);
            }
        }

        /// <summary>
        /// The _iteration counter
        /// </summary>
        private int _iterationCounter = 0;
        /// <summary>
        /// Results the changed.
        /// </summary>
        /// <param name="observableResult">The observable result.</param>
        /// <param name="velocityPoint">The velocity point.</param>
        /// <param name="accelerationPoint">The acceleration point.</param>
        /// <param name="forcePoint">The force point.</param>
        public void ResultChanged(ObservableProgress observableResult, Point2D velocityPoint, Point2D accelerationPoint, Point2D forcePoint)
        {
            this.result = observableResult;
            AddVelocity(velocityPoint);
            AddAcceleration(accelerationPoint);
            AddForce(forcePoint);
            _iterationCounter++;
        }

        /// <summary>
        /// Adds the force.
        /// </summary>
        /// <param name="forcePoint">The force point.</param>
        private void AddForce(Point2D forcePoint)
        {
            ObservableCollection<Point2D> tempForce = new ObservableCollection<Point2D>();
            tempForce = ForceCollection;

            double force = Math.Sqrt(Math.Pow(forcePoint.X,2) + Math.Pow(forcePoint.Y,2));

            if (tempForce.Count() > 9)
            {
                tempForce.RemoveAt(0);
                tempForce.Add(new Point2D(_iterationCounter, force));
            }
            else
            {
                tempForce.Add(new Point2D(_iterationCounter, force));
            }
            //Is done to be able to update the ForceCollection using RaisePropertyChange
            //The entire property has to change to Raise property change.
            ForceCollection = tempForce;
        }

        /// <summary>
        /// Adds the acceleration.
        /// </summary>
        /// <param name="accelerationPoint">The acceleration point.</param>
        private void AddAcceleration(Point2D accelerationPoint)
        {
            ObservableCollection<Point2D> tempAcceleration = new ObservableCollection<Point2D>();
            tempAcceleration = AccelerationCollection;

            double acceleration = Math.Sqrt(Math.Pow(accelerationPoint.X, 2) + Math.Pow(accelerationPoint.Y, 2));

            if (tempAcceleration.Count() > 9)
            {
                tempAcceleration.RemoveAt(0);
                tempAcceleration.Add(new Point2D(_iterationCounter, acceleration));
            }
            else
            {
                tempAcceleration.Add(new Point2D(_iterationCounter, acceleration));
            }
            //Is done to be able to update the AccelerationCollection using RaisePropertyChange
            //The entire property has to change to Raise property change.
            AccelerationCollection = tempAcceleration;
        }

        /// <summary>
        /// Adds the velocity.
        /// </summary>
        /// <param name="velocityPoint">The velocity point.</param>
        private void AddVelocity(Point2D velocityPoint)
        {
            ObservableCollection<Point2D> tempVelocity = new ObservableCollection<Point2D>();
            tempVelocity = VelocityCollection;

            double velocity = Math.Sqrt(Math.Pow(velocityPoint.X, 2) + Math.Pow(velocityPoint.Y, 2));

            if (tempVelocity.Count() > 9)
            {
                tempVelocity.RemoveAt(0);
                tempVelocity.Add(new Point2D(_iterationCounter, velocity));
            }
            else
            {
                tempVelocity.Add(new Point2D(_iterationCounter, velocity));
            }
            //Is done to be able to update the AccelerationCollection using RaisePropertyChange
            //The entire property has to change to Raise property change.
            VelocityCollection = tempVelocity;
        }
    }
}
